from tw.api import Widget, JSLink, CSSLink, Link
from tw.core.resources import registry, working_set, Requirement
from tw.mootools.defaults import __MOOTOOLS_VERSION__, __DEFAULT_MODULE__, __DEFAULT_SUFFIX__

import tw

import os

#__all__ = ["Twtools"]

class DomElementWithValue(object):
    def __init__(self, id, value, attrs={}):
        self.id = id
        self.attrs = attrs
        self.value = value

class MootoolsLink(Link):
    params = {'basename': '(string) basename for the given file.  if you want mootools-core.js, the base is mootools',
              'suffix': '(string) yc (yui compressed), jm (jsmin compressed) or nc (not compressed).  Default is "yc"',
              'part': '(string) core or more.  Default is "core"',
              'version': '(string) select the mootools version you would like to use. Default version is: '+__MOOTOOLS_VERSION__,
    }

    version = __MOOTOOLS_VERSION__
    modname = "tw.mootools"
    extension = 'js'
    part = __DEFAULT_MODULE__
    suffix = __DEFAULT_SUFFIX__

    def __init__(self, *args, **kw):
        super(Link, self).__init__(*args, **kw)
        if not self.is_external:
            modname = self.modname or self.__module__
            self.webdir, self.dirname, self.link = registry.register(
                self, modname, self.filename
                )
            
        if 'suffix' in kw:
            self.suffix = kw['suffix']
            
        if 'part' in kw:
            self.part = kw['part']

    def _get_link(self):
        if self.is_external:
            return self.external_link
        return tw.framework.url(self._link or '')

    def _set_link(self, link):
        self._link = link

    link = property(_get_link, _set_link)

    def abspath(self, filename):
        return os.sep.join((os.path.dirname(__file__), filename))

    def try_filename(self, filename):
        abspath = self.abspath(filename)
        if os.path.exists(abspath):
            return filename
        return False

    @property
    def filename(self):
        #make basename windows/qnix compat
        basename = os.sep.join(('static', self.version, 'mootools'))
        #try the default
        filename =  basename+'-'+self.version+'-'+self.part+'-'+self.suffix+'.'+self.extension
        return filename

class MootoolsCoreLink(JSLink, MootoolsLink):
    pass
class MootoolsMoreLink(JSLink, MootoolsLink):
    part = 'more'
    
## JS dependencies can be listed at 'javascript' so they'll get included
## before
moo_core_js    = MootoolsCoreLink(suffix='nc')
moo_core_js_compressed    = MootoolsCoreLink(suffix='yc')

moo_more_js    = MootoolsMoreLink(suffix='nc')
moo_more_js_compressed    = MootoolsMoreLink(suffix='yc')

# for backward compat
moo_js = moo_core_js
moo_js_compressed = moo_core_js_compressed

    
class MooTools(Widget):
    template = """<span xmlns:py="http://genshi.edgewall.org/" py:strip="true"></span>"""
    
    def __init__(self, version=None):
        self.version = version
    
    @property
    def javascript(self):
        if self.version:
            return [
                  MootoolsCoreLink(version=self.version),
                  MootoolsMoreLink(version=self.version),
                 ]
        else:
            return [
                  MootoolsCoreLink(),
                  MootoolsMoreLink(),
                 ]

# for backward compat
MooToolsCompressed = MooTools


from datetime import datetime
import logging
import errno

import tw
from tw.api import CSSLink, JSLink, js_function
from tw.forms import FormField, validators

from tw.mootools.base import moo_core_js_compressed, moo_more_js_compressed

calendar_js = JSLink(modname=__name__,
                      filename='static/datepicker/DatePicker.js',
                      javascript=[])
calendar_css = CSSLink(modname=__name__,
                        filename='static/datepicker/DatePicker.css',
                        javascript=[])

class CalendarDatePicker(FormField):
    """
    Uses a javascript calendar system to allow picking of calendar dates.
    The date_format is in mm/dd/yyyy unless otherwise specified
    """
    css = [calendar_css]
    javascript = [moo_core_js_compressed, calendar_js]
    template = "tw.mootools.templates.datepicker"
    params = [
        "calendar_lang", "not_empty", "date_format"
        ]
    calendar_lang = 'en'
    not_empty = True
    date_format = "mm/dd/yyyy"
    validator = None

    _default = None

    def __init__(self, *args, **kw):
        super(CalendarDatePicker, self).__init__(*args, **kw)
        if self.default is None and self.not_empty:
            self.default = lambda: datetime.now()
        self.validator = self.validator or validators.DateTimeConverter(
            format=self.transcode_date_format(self.date_format),
            not_empty=self.not_empty
            )

    def transcode_date_format(self, value):
        return value.replace(
                            'mm','%m'
                            ).replace(
                            'dd', '%d'
                            ).replace(
                            'yyyy', '%Y'
                            ).replace(
                            'yy', '%y'
                            )
    
    def get_calendar_lang_file_link(self, lang):
        """
        Returns a CalendarLangFileLink containing a list of name
        patterns to try in turn to find the correct calendar locale
        file to use.
        """
        fname = 'static/calendar/lang/calendar-%s.js' % lang.lower()
        return JSLink(modname='tw.forms',
                      filename=fname,
                      javascript=self.javascript)

    def update_params(self, d):
        super(CalendarDatePicker, self).update_params(d)
        try:
            d.strdate = d.value.strftime(self.transcode_date_format(d.date_format))
        except AttributeError:
            d.strdate = d.value
        d.css_classes.append('DatePicker')



from tw.api import Widget, JSLink, CSSLink, CSSSource
from genshi.template.text import TextTemplate
import tw.forms

from tw.mootools.base import moo_core_js_compressed, moo_more_js_compressed

# declare your static resources here
omnigrid_js = JSLink(modname=__name__, filename='static/omnigrid/omnigrid11.js', javascript=[])
omnigrid_css = CSSLink(modname=__name__, filename='static/omnigrid/omnigrid.css', javascript=[])

class SimpleGridWidget(tw.forms.datagrid.DataGrid):
    ## You can also define the template in a separate package and refer to it
    ## using Buffet style uris
    template = "genshi:tw.mootools.templates.simplegrid"

    javascript = [moo_core_js_compressed, moo_more_js_compressed, omnigrid_js]
    css = [omnigrid_css]
    



from tw.mootools.base import (moo_core_js, moo_core_js_compressed,
                              moo_more_js, moo_more_js_compressed,
                              moo_js, moo_core_js_compressed,
                              MooTools, MooToolsCompressed, MootoolsLink,
                              MootoolsCoreLink, MootoolsMoreLink)
from tw.mootools.dates import CalendarDatePicker


# Metadata for the widget EGG. Should leave it for the (unimplemented) widget
# tracker to track it

__PROJECT__ = 'tw.mootools'
__DESCRIPTION__ = 'ToscaWidgets wrapper for Mootools Javascript Toolkit'
__URL__ = 'http://toscawidgets.org/'
__VERSION__ = '0.9.8.1'
__AUTHOR__ = 'Christopher Perkins, Jonathan Schemoul'
__EMAIL__ = 'chris@percious.com, jonathan.schemoul@jmsinfor.com'
__COPYRIGHT__ = "Copyright 2008-2009 Christopher Perkins, Jonathan Schemoul"
__LICENSE__ = 'MIT'



from tw.api import Widget, JSLink, CSSLink, CSSSource, JSSource
from genshi.template.text import TextTemplate
from base import DomElementWithValue, moo_core_compressed_js

# declare your static resources here
widget_js = JSLink(modname=__name__, filename='static/sortable.js', javascript=[])

class SortableWidget(Widget):
    ## You can also define the template in a separate package and refer to it
    ## using Buffet style uris
    template = "genshi:tw.mootools.templates.sortable"

    javascript = [moo_core_js]

    #attributes for the entire widget
    attrs = {'width':'300px', 'backround-color':'white'}

    #attributes that are the same across blocks
    blockAttrs = {}
    
    params = ['attrs', 'blockAttrs', 'widget_css', 'widget_js', 'onComplete']

    onComplete = "console.log('got to onComplete')"
    
    def update_params(self, d):
        self._my_update_params(d)
        Widget.update_params(self, d)
        return d

    def _my_update_params(self, d):
        values = d['value']
        d['widget_css'] = [CSSSource(self._generateCSS(values)),]
        return d
    
    cssTemplate = """${"#"}${id} { 
	position: inherit;
}
 
ul${"#"}sortables {
	width: 300px;
	margin: 0;
	padding: 0;
#for a, v in attrs.iteritems()
    ${a} : ${v};
#end
}
 
#for value in values
    li.sortable_${value.id} {
            padding: 4px 8px;
            cursor: pointer;
            list-style: none;
    #for a, v in value.attrs.iteritems()
            ${a} : ${v};
    #end
    }
#end
 
ul${"#"}sortables li {
	margin: 10px 0;
}


"""
    
    def _generateCSS(self, values):
        template = TextTemplate(self.cssTemplate)
        s = template.generate(values=values, blockAttrs=self.blockAttrs, attrs=self.attrs, id=self.id).render()
        return s
    



from toscawidgets.api import Widget, JSLink, CSSLink, CSSSource
from genshi.template.text import TextTemplate

from tw.mootools.base import moo_core_js_compressed, moo_js, moo_more_js

# declare your static resources here
kwick_js = JSLink(modname=__name__, filename='static/kwicks.js', javascript=[])

class KwickWidget(Widget):
    ## You can also define the template in a separate package and refer to it
    ## using Buffet style uris
    template = "genshi:tw.mootools.templates.kwick"

    javascript = [moo_core_js_compressed, kwick_js]
    
    attrs = {'height':'100px', 'backround-color':'white'}
    blockAttrs = {'height':'100px', 'width':'117px', 'backround-color':'white'}
    params = ['attrs', 'blockAttrs', 'moo_css']

    def update_params(self, d):
        self._my_update_params(d)
        Widget.update_params(self, d)
        return d

    def _my_update_params(self, d):
        values = d['value']
        d['moo_css'] = [CSSSource(self._generateCSS(values)),]
        return d
    
    cssTemplate = """${"#"}kwicks_container { 
    background-color: ${attrs.backgroundColor};
    height: ${attrs['height']};
}
${"#"}kwicks {
    position: relative;
    float: left;
    display: block;

}
 
${"#"}kwicks .kwick {
    #for a, v in attrs.iteritems()
        $a : $v ;
    #end
}
  
#for value in values
${"#"}kwick_${value.id} {
    float: left;
    display: block;
    #for a, v in value.attrs.iteritems()
         $a : $v;
    #end
    }
#end
"""
    
    def _generateCSS(self, values):
        template = TextTemplate(self.cssTemplate)
        s = template.generate(values=values, attrs=self.attrs, blockAttrs=self.blockAttrs).render()
        return s
    



import tw.forms
import tw.api
from tw.api import Widget, JSLink, CSSLink, CSSSource

from tw.forms import InputField, InputWidget, CalendarDatePicker

from tw.mootools.base import moo_core_js_compressed, moo_more_js_compressed

formcheck_js = JSLink(modname=__name__,
                      filename='static/formcheck/formcheck.js',
                      javascript=[])
formcheck_css = CSSLink(modname=__name__,
                        filename='static/formcheck/theme/classic/formcheck.css',
                        javascript=[])

twmootoolsforms_js = JSLink(modname=__name__,
                      filename='static/forms.js',
                      javascript=[])

def set_widgets_validator_classes(widget):
    validator_name = widget.validator.__class__.__name__
    if validator_name == 'All':
        validator_names = widget.validator.validators
    else:
        validator_names = [validator_name]
        
    for validator in validator_names:
        validator_name = widget.validator.__class__.__name__
        if validator_name == 'MinLength':
            validator_name += " validate['length[%d,-1]']" % validator.minLength
            
        if validator_name == 'NotEmpty' or (hasattr(widget, 'is_required') and widget.is_required):
            validator_name += " validate['required']"
            
        if validator_name == 'Int':
            validator_name += " validate['digit']"
            
        widget.css_classes.append(validator_name)
        
    if widget.attrs.get('class'):
        widget.attrs['class'] += ' ' + validator_name
    if hasattr(widget, 'children'):
        for child in widget.children:
            set_widgets_validator_classes(child)

class CustomisedForm(tw.forms.Form):
    """A form that allows specification of several useful client-side behaviours.
    Inherits all features of tw.dynforms.CustomisedForms plus client side validation"""
    params = {
        'blank_deleted': 'Blank out any deleted form fields from GrowingTable on the page. This is required for growing to function correctly - you must use GrowingTableFieldSet within a CustomisedForm with this option set.',
        # TBD: 'save_prompt': 'If the user navigates away without submitted the form, and there are changes, this will prompt the user.',
        'disable_enter': 'Disable the enter button (except with textarea fields). This reduces the chance of users accidentally submitting the form.',
        'validate_inputs': 'When the user clicks the submit button, verify that all required fields are entered'
    }
    blank_deleted = True
    #save_prompt = True
    disable_enter = True
    validate_inputs = True
    javascript = [moo_core_js_compressed,
                  moo_more_js_compressed,
                  formcheck_js,
                  twmootoolsforms_js]
    css = [formcheck_css]
    form_lang = 'en'
    
    
    def __init__(self, *args, **kwargs):
        super(CustomisedForm, self).__init__(*args, **kwargs)
        #set_widgets_validator_classes(self)
        
    def get_form_lang_file_link(self, lang):
        """
        Returns a CalendarLangFileLink containing a list of name
        patterns to try in turn to find the correct calendar locale
        file to use.
        """
        fname = 'static/formcheck/lang/%s.js' % lang
        return JSLink(modname=__name__,
                      filename=fname,
                      javascript=self.javascript)
    
    
    def update_params(self, params):
        super(CustomisedForm, self).update_params(params)
        if params.get('validate_inputs', self.blank_deleted):
            params.css_classes.append('validate_form')
            
        if params.get('blank_deleted', self.blank_deleted):
            params.setdefault('attrs', {})['onsubmit'] = 'twd_blank_deleted()'
        if params.get('disable_enter', self.disable_enter):
            self.add_call('document.onkeypress = twd_suppress_enter;')
        
        set_widgets_validator_classes(self)
        
        self.get_form_lang_file_link(self.form_lang).inject()


import os
import sys
from fnmatch import fnmatchcase
from distutils.util import convert_path

from setuptools import setup, find_packages


def find_package_data( package='', where='.', only_in_packages=True):
    """Finds static resources in package. Adapted from turbogears.finddata."""  
    out = {}
    exclude = ('*.py', '*.pyc', '*~', '.*', '*.bak', '*.swp*')
    exclude_directories = ('.*', 'CVS', '_darcs', './build',
                           './dist', 'EGG-INFO', '*.egg-info')
    stack = [(convert_path(where), '', package, only_in_packages)]
    while stack:
        where, prefix, package, only_in_packages = stack.pop(0)
        for name in os.listdir(where):
            fn = os.path.join(where, name)
            if os.path.isdir(fn):
                bad_name = False
                for pattern in exclude_directories:
                    if (fnmatchcase(name, pattern)
                        or fn.lower() == pattern.lower()):
                        bad_name = True
                        print >> sys.stderr, (
                            "Directory %s ignored by pattern %s"
                            % (fn, pattern))
                        break
                if bad_name:
                    continue
                if os.path.isfile(os.path.join(fn, '__init__.py')):
                    if not package:
                        new_package = name
                    else:
                        new_package = package + '.' + name
                    stack.append((fn, '', new_package, False))
                else:
                    stack.append((fn, prefix + name + '/', package, only_in_packages))
            elif package or not only_in_packages:
                # is a file
                bad_name = False
                for pattern in exclude:
                    if (fnmatchcase(name, pattern)
                        or fn.lower() == pattern.lower()):
                        bad_name = True
                        print >> sys.stderr, (
                            "File %s ignored by pattern %s"
                            % (fn, pattern))
                        break
                if bad_name:
                    continue
                out.setdefault(package, []).append(prefix+name)
    return out

execfile(os.path.join("tw", "mootools", "release.py"))

setup(
    name=__PROJECT__,
    version=__VERSION__,
    description=__DESCRIPTION__,
    author=__AUTHOR__,
    author_email=__EMAIL__,
    url=__URL__,
    download_url='http://toscawidgets.org/download',
    install_requires=[
        "ToscaWidgets>0.8",
        ## Add other requirements here
        # "Genshi",
        ],
    packages=find_packages(exclude=['ez_setup', 'tests']),
    namespace_packages = ['tw'],
    zip_safe=False,
    include_package_data=True,
    test_suite = 'nose.collector',
    package_data= find_package_data('tw.mootools'),
    entry_points="""
        [toscawidgets.widgets]
        # Use 'widgets' to point to the module where widgets should be imported
        # from to register in the widget browser
        widgets = tw.mootools
        # Use 'samples' to point to the module where widget examples
        # should be imported from to register in the widget browser
        samples = tw.mootools.samples
        # Use 'resources' to point to the module where resources
        # should be imported from to register in the widget browser
        #resources = tw.mootools.resources
    """,
    keywords = [
        'toscawidgets.widgets',
    ],
    classifiers = [
        'Development Status :: 4 - Beta',
        'Environment :: Web Environment',
        'Environment :: Web Environment :: ToscaWidgets',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Topic :: Software Development :: Widget Sets',
        'Intended Audience :: Developers',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
    ],
)



from tw.api import Widget, JSLink, CSSLink, CSSSource
from genshi.template.text import TextTemplate
import tw.forms

from tw.mootools.base import moo_core_js_compressed, moo_more_js_compressed

# declare your static resources here
calendar_js = JSLink(modname=__name__, filename='static/calendar/mooECal.js', javascript=[])
calendar_css = CSSLink(modname=__name__, filename='static/calendar/style.css', javascript=[])

class CalendarWidget(Widget):
    """
    To use this calendar, just instanciate it. This is a prototype, so no advanced options available.
    Value should be an array of dictionnary including :
    * "title" (string)
    * "start" (date or datetime)
    * "end" (date or datetime)
    * "location" (string)
    * "url" (string, optionnal)
    Example of use :
    ${CalendarWidget().display([
        {
            'title':'Get Groceries',
            'start':date(2008,12,05),
            'end':date(2008,12,06),
            'location':'Store',
            'url':'http://www.myshoppingmal.com/'
        },
        {
            'title':'Goin Cow Tip',
            'start':datetime(2008,12,20,23,30),
            'end':'datetime(2008,12,20,23,45),
            'location':'',
            'url':'/ive-won/'
        },
        {
            'title':'Hair Cut',
            'start':datetime(2008,12,22,10,0),
            'end':datetime(2008,12,22,11,0),
            'location':'at the hair-dresser'
        }
    ])}
    """
    template = "genshi:tw.mootools.templates.calendar"

    javascript = [moo_core_js_compressed, moo_more_js_compressed, calendar_js]
    css = [calendar_css]
    
